#' Get objective function
#' 
#' This function receives a configuration for a tuning experiment,
#' and returns an objective function to be tuned.
#'
#' @param config list, e.g., generated by \code{\link{get_exp_config}}.
#' @param timeout integer, time in seconds after which an evaluation will be aborted.
#' 
#' @return an objective function that can be optimized via spot
#' 
#' @export
#' @import callr
get_objf <- function(config,timeout=60,penalty=Inf){
  force(config)
  force(timeout)
  
  objfun <- function(x,seed=NA){
    params <- as.list(x)
    ## set parameter names
    names(params) <- config$tunepars
    
    ## transform to actual parameter scale
    for(i in 1:length(params)){
      params[[i]] <- config$transformations[[i]](params[[i]])
    }
    
    ## convert integer levels to factor levels for categorical params
    if(length(config$factorlevels)>0)
      params <- int2fact(params,config$factorlevels)
    
    ## set fixed parameters (which are not tuned, but are also not set to default value)
    if(length(config$fixpars)>0)
      params <- c(params,config$fixpars)
    
    ## set parameters in relation to other parameters
    nrel <- length(config$relpars)
    if(nrel>0){
      for(i in 1:nrel){
        params[names(config$relpars)[i]] <- with(params,eval(config$relpars[[i]]))
      }
    }

    timestamp1 <- as.numeric(Sys.time())
    params$seed <- seed
    ## call the evaluation in a new process via callr
    ## this ensures that the call can be timed-out even when stuck in non-R low-level code
    if(is.na(timeout)){
			res <- do.call(config$run,params)
    }else{
      res <- try(
        r(
          function(fun,params){
            do.call(fun,params)
          },
          timeout=timeout,
          args = list(params=params,fun=config$run), 
          poll_connection = FALSE,
          package="keras"
        )
      )
    }
    
    timestamp2 <- as.numeric(Sys.time())
		
		duration <- timestamp2-timestamp1
    
    ## determine a value to return in case of errors or timeouts.
    if( class(res)[1] == "try-error"){ 
      res <- penalty
    }
    return(matrix(c(res,duration),1))
  }
  force(objfun)
  objvecf <- function(x,seed=NaN){
    res <- NULL
    for(i in 1:nrow(x))
      res <- rbind(res,objfun(x[i,,drop=FALSE],seed[i]))
    return(res)
  }
}


