#' @title Check validity of input parameters
#'
#' @param chars character
#'
#' @export
#' @keywords internal
valid_inputs <- function(chars) {
	is_valid <- NULL
	for (i in 1:length(chars)) {
		ch <- chars[i]
		valid <- FALSE
		if (is.list(.GlobalEnv[[ch]])) {
			if (!is.null(.GlobalEnv[[ch]]$modelFit)) {
				valid <- TRUE
			}
		}
		is_valid <- c(is_valid, valid)
	}
	chars[is_valid]
}


#' @title Helper function: translate levels
#'
#' @description 
#' Translate existing levels of a factor into new levels.
#'
#' @param x a factor vector to be translated
#' @param translations a named list that specifies the translation: \code{list(newlevel=c(oldlevel1,oldlevel2,etc))}.
#'
#' @export
translate_levels <- function(x, translations) {
	x <- as.character(x)
	for (i in 1:length(translations)) {
		sel <- x %in% translations[[i]]
		x[sel] <- names(translations[i])
	}
	as.factor(x)
}

#' @title Helper function: transform integer to factor
#'
#' @description 
#' This function re-codes a factor with pre-specified factor levels, using an integer encoding as input.
#'
#' @param x an integer vector (that represents factor vector) to be transformed
#' @param lvls the original factor levels used
#' @return the same factor, now coded with the original levels
#'
#' @export
int2fact <- function(x, lvls) {
	nms <- names(lvls)
	for (i in 1:length(lvls)) {
		x[[nms[i]]] <-  lvls[[i]][x[[nms[i]]]]
	}
	x
}


#' @title Identity transformation
#'
#' @param x input
#' @return \code{x}
#'
#' @export
trans_id <- function(x) {
	x
}

#' @title 2 power x transformation
#'
#' @param x input
#' @return \code{2^x}
#'
#' @export
trans_2pow <- function(x) {
	2 ^ x
}

#' @title 2 power x transformation with round
#'
#' @param x input
#' @return \code{round(2^x)}
#'
#' @export
trans_2pow_round <- function(x) {
	round(2 ^ x)
}

#' @title 10 power x transformation
#'
#' @param x input
#' @return \code{10^x}
#'
#' @export
trans_10pow <- function(x) {
	10 ^ x
}


#' @title 10 power x transformation with round
#'
#' @param x input
#' @return \code{round(10^x)}
#'
#' @export
trans_10pow_round <- function(x) {
	round(10 ^ x)
}


#' @title Make benchmark
#'
#' @importFrom benchmarkme benchmark_std
#'
#' @export
#' @keywords internal
makeBench <- function() {
	res <- benchmark_std(runs = 1, verbose = FALSE)
	total_time <- sum(res$user) ## in seconds
	return(total_time)
}


#' @title Get budget (time, function evaluations)
#' @description
#' Determine time factor and corrected  budget&timeout.
#' Runs a benchmark to determine processing speed of assigned computing resources
#'
#' @param runtime_bench runtime
#' @param settings settings
#'
#' @return settings
#' @export
#' @keywords internal
getBudget <- function(runtime_bench,
											settings) {
	if (is.null(runtime_bench)) {
		runtime_bench <- mean(replicate(3, makeBench())) ## in seconds
		runtime_default <- 5.6 ## in seconds
		time_multiplier <- runtime_bench / runtime_default
	}
	settings$actual_timebudget <-	time_multiplier * settings$timebudget
	settings$actual_timeout <- time_multiplier * settings$timeout
	settings$runtime_bench <- runtime_bench
	settings$time_multiplier <- time_multiplier
	return(settings)
}
