## If not installed, do:
# install.packages("reticulate")
# install.packages("dplyr")
# install.packages("xgboost")
## Load the packages:
require(reticulate) 
require(dplyr) 

## Load a few R functions required for this example
source("getobjf.R")

source("misc.R")

source("plotting.R")

source("runXgboost.R")

source("get_data.R")

## configuration for the tuning problem
cfg <- list(
		## Names of the parameters
    tunepars = c("max_depth","nrounds","eta","colsample_bytree"),
    ## their lower bounds
		lower =    c(1,           1,          -2,     0.1),
    ## their upper bounds
		upper =    c(10,          100,         0,         1  ), 
		## their type
    type =    c("integer",  "integer", "numeric", "numeric"),
		## transformations to apply
    transformations = 
					 c(trans_id,    trans_id,    trans_10pow,    trans_id),
		## another parameter that will not be tuned, but is fixed					 
    fixpars = list(),
		## not used in this example (specify levels of categorical parameters):
    factorlevels = list(),
		## not used in this example (specify parameters that are relative to other parameters)
		relpars = list(),
		run=runXgboost
)

## penalty value if modeling fails
penalty <- sd(trainval_label)

## create an objective function that contains the parameterized learning process
objf <- get_objf(cfg,
                 NA, # no timeout
                 penalty) # and the penalty

## TEST: 
#objf(matrix(cfg$lower,1),1)
#objf(matrix(cfg$lower,1),2)
#objf(matrix(cfg$upper,1),1)
#objf(matrix(cfg$upper,1),2)

## install SPOT:
#install.packages("SPOT")
## load SPOT package
require(SPOT)

## time limit for tuning
timebudget <- 60 ## minutes
# Note: if this runs on a rather slow computer, 60 minutes might not be sufficient.

resultfile <- "resultFilexgboost.RData"

if(!file.exists(resultfile)){
  ## Run tuning with SPOT
  result <- spot(fun = objf,
               lower=cfg$lower,
               upper=cfg$upper,
               control = list(types=cfg$type,
                              maxTime = timebudget, # time budget
                              plots=T, # do progress plots
                              optimizer=optimDE, # optimize surrogate with DE
                              noise=TRUE, # problem is noisy
                              seedSPOT=1, # initial RNG seed for SPOT
                              seedFun=1, # initial, incremental seed for evaluation
                              designControl=list(size=5*length(cfg$lower),replicates=3), # initial design
                              replicates=10, # evaluate each candidate several times
                              funEvals=Inf, # no limit on number of evaluations (only maxTime)
                              model=buildKriging, #Kriging / Gaussian process surogate
                              modelControl=list(target="ei",useLambda=T,reinterpolate=T), #Kriging configuration
                              optimizerControl=list(funEvals=400*length(cfg$lower)) # evaluation budget for DE
               )
          )

  ## Perform repetitions with final solution, but using test data.
  cfg$fixpars <- list(eval="test")
  objf2 <- get_objf(cfg,NA,penalty)
  ## Merging the data from replications
  df <- data.frame(x=result$x,yrep=result$y,logInfo=result$logInfo)
  df <- df %>% group_by(across(starts_with("x."))) 
  df <- df %>% summarise(y=mean(yrep), 
                   sd=sd(yrep),
                   logInfo=mean(logInfo),
                   pos=first(cur_group_rows()))
  df <- df %>% arrange(pos)
  df$pos <- NULL
  ## validate
  best_real_x <- df[which.min(df$y),1:length(cfg$upper)]
  best_pred_x <- result$x[which.min(predict(result$modelFit,result$x)$y),,drop=F]
  best <- best_pred_y <- NULL
  for(i in 1:10){
    best <- c(best, objf2(best_real_x,i)[1])
    best_pred_y <- c(best_pred_y, objf2(best_pred_x,i)[1])
  }
  
  ## Preprocess the result data for visual evaluation
  result2 <- result
  result2$x <- as.matrix(select(df,starts_with("x.")))
  result2$y <- as.matrix(df[,"y",drop=F])
  result2$logInfo <- as.matrix(df[,c("logInfo","sd"),drop=F])
  resultpp <- prepare_spot_result_plot(data=result2,
                                       model=buildRandomForest,
                                       modelControl=list(types=cfg$type),
                                       namesx=cfg$tunepars,
                                       namesy="rmse",
                                       nameslog=c("duration","st.dev.")
  )
  resultpp2 <- prepare_spot_result_plot(data=result2,
                                        model=buildKriging,
                                        modelControl=list(types=cfg$type,useLambda=T,reinterpolate=F),
                                        namesx=cfg$tunepars,
                                        namesy="rmse",
                                        nameslog=c("duration","st.dev.")
  )
  
  rm(result2)
  
  ## save result in RData file
  save(resultpp,resultpp2,result,best,best_real_x,best_pred_x,best_pred_y,file=resultfile)
}else{
  load(resultfile)
}

## interactive plots
require(plotly)
require(ggsci)
require(emoa)

## surface plots via surrogate model
plot_surface(resultpp2,which = c(1,2))
plot_surface(resultpp2,which = c(1,3))
plot_surface(resultpp2,which = c(1,4))
plot_surface(resultpp2,which = c(2,3))
plot_surface(resultpp2,which = c(2,4))
plot_surface(resultpp2,which = c(3,4))

## sensitivity
plot_sensitivity(resultpp2)

## parallel plot of the actual data
plot_parallel(resultpp2)
plot_parallel(resultpp2,yrange = c(1,3.5))

## Plot of pareto optimal data (wrt reward + duration)
plot_pareto(resultpp2)

## Plot of pareto optimal data (wrt reward + st.dev. of reward)
plot_pareto(resultpp2,yvar = 3)
